// -*- C++ -*-
/*!
 * @file  CUI.cpp
 * @brief Demo CUI component
 * $Date$
 *
 * $Id$
 */

#include "CUI.h"

#include "sys/types.h"
#include "dirent.h"

/*----------------- [eBeB֐` -----------------*/
static int getString( char* sBuf );  // [U̓ () 擾
static short getShort( void );       // [U̓ () 擾
static double getDouble( void );     // [U̓ (l) 擾



// Module specification
// <rtc-template block="module_spec">
static const char* cui_spec[] =
  {
    "implementation_id", "CUI",
    "type_name",         "CUI",
    "description",       "Demo CUI component",
    "version",           "0.1",
    "vendor",            "",
    "category",          "Generic",
    "activity_type",     "COMMUTATIVE",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

CUI::CUI(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_SizeIn("Size", m_Size),
    m_WaveIn("Wave", m_Wave),
    m_ResultIDIn("ResultID", m_ResultID),
    m_TrustyIn("Trusty", m_Trusty),
    m_TrigegerOut("Trigeger", m_Trigeger),
    m_FileOut("File", m_File),
    m_SendSizeOut("SendSize", m_SendSize),
    m_SendWaveOut("SendWave", m_SendWave),
    m_IDOut("ID", m_ID),
    m_ParamDirOut("ParamDir", m_ParamDir),
    m_TalkerMngPort("TalkerMng"),m_PowerInfoPort("PowerInfo"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Size", m_SizeIn);
  registerInPort("Wave", m_WaveIn);
  registerInPort("ResultID", m_ResultIDIn);
  registerInPort("Trusty", m_TrustyIn);
  
  // Set OutPort buffer
  registerOutPort("Trigeger", m_TrigegerOut);
  registerOutPort("File", m_FileOut);
  registerOutPort("SendSize", m_SendSizeOut);
  registerOutPort("SendWave", m_SendWaveOut);
  registerOutPort("ID", m_IDOut);
  registerOutPort("ParamDir", m_ParamDirOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_TalkerMngPort.registerConsumer("TalkerMng0", "TalkerMng", m_TalkerMng0);
  m_PowerInfoPort.registerConsumer("PowerInfo0", "PowerInfo", m_PowerInfo0);
  
  // Set CORBA Service Ports
  registerPort(m_TalkerMngPort);
  registerPort(m_PowerInfoPort);
  
  // </rtc-template>

}

CUI::~CUI()
{
}


RTC::ReturnCode_t CUI::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t CUI::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CUI::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CUI::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CUI::onActivated(RTC::UniqueId ec_id)
{
  // |[gɂ݂Ă\邽߁A
  // xAǂݍݏs
  m_SizeIn.read();
  m_WaveIn.read();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CUI::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

enum{
  STATE_NORMAL  = 0,
  STATE_VOICEINPUT_WAITING,
  STATE_RESULT_WAITING,
};

enum{
  USER_SELECT_SETUP_CONFIG  = 0,
  USER_SELECT_GET_VOICE,
  USER_SELECT_GET_FILE
};

RTC::ReturnCode_t CUI::onExecute(RTC::UniqueId ec_id)
{
  static int iStat = STATE_NORMAL;
  static int iTalker = 0;
  int        iSel;

  switch( iStat ) {
  //----- ʏԂ̏ꍇ ------------------------------------
  case STATE_NORMAL:
    {
      // [UɁA͂𑣂
      fprintf( stdout, "<Select>\n" );
      fprintf( stdout, "  %d:Set up config\n", USER_SELECT_SETUP_CONFIG );
      fprintf( stdout, "  %d:Get voice\n", USER_SELECT_GET_VOICE );
      fprintf( stdout, "  %d:Get file\n", USER_SELECT_GET_FILE );
      fprintf( stdout, "    Input no: " );
      // [ÚA͂擾
      iSel = getShort();
      // ݒs
      if( iSel == USER_SELECT_SETUP_CONFIG ) {
        onSetParam();
      }
      // ̃}CN͂s
      else if( iSel == USER_SELECT_GET_VOICE ) {
        // Jn
        iTalker = onStart();
        // ͑҂ԂɑJ
        iStat = STATE_VOICEINPUT_WAITING;
      }
      // t@C͂s
      else if( iSel == USER_SELECT_GET_FILE ) {
        // Jn
        iTalker = onStartFile();
        // ͑҂ԂɑJ
        iStat = STATE_VOICEINPUT_WAITING;
      }
    }
    break;
  //----- ͑҂Ԃ̏ꍇ ----------------------------
  case STATE_VOICEINPUT_WAITING:
    {
      // f[^̓|[gɁAf[^TCY͂ꂽꍇ
      if( m_SizeIn.isNew() ) {
        // f[^擾
        if( onGetSound( iTalker ) )
          // ʓ͑҂ԂɑJ
          iStat = STATE_RESULT_WAITING;
        else
          // ʏԂɖ߂
          iStat = STATE_NORMAL;
      }
    }
    break;
  //----- ʓ͑҂Ԃ̏ꍇ --------------------------
  case STATE_RESULT_WAITING:
    {
      // f[^̓|[gɁAʍڐ͂ꂽꍇ
      if( m_ResultIDIn.isNew() ) {
        // ʂ擾
        onGetResult();
        // ʏԂɖ߂
        iStat = STATE_NORMAL;
      }
    }
    break;
  //----- ̑ --------------------------------------------
  default:
    break;
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CUI::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CUI::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CUI::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CUI::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CUI::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



/*
 @brief   bҎʃVXe̐ݒύX
 @author  AdIn Research,Inc
 @date    2007/4/11
*/
void CUI::onSetParam( void )
{
  char   sBuf[256];
  char   sBuf1[256];
  short  iWF, iOut, iWF1, iOut1;
  double dLow, dUp, dLow1, dUp1;

  // o (Charact) R|[lgA݂̐ݒ擾
  try {
    m_PowerInfo0->GetParam( iWF, dLow, dUp, iOut );
  }
  catch(...) {
    fprintf( stdout, "Error in service port( PowerInfo ), check connection.\n" );
    return;
  }

  // ֐̐ݒl擾
  fprintf( stdout, "Input window function id(%d) -> ", iWF );
  iWF1 = getShort();
  // g̐ݒl擾
  fprintf( stdout, "Input lower limit freq(%lf) -> ", dLow );
  dLow1 = getDouble();
  // g̐ݒl擾
  fprintf( stdout, "Input upper limit freq(%lf) -> ", dUp );
  dUp1 = getDouble();
  // gш (g-g) ̕擾
  fprintf( stdout, "Input devide count(%d) -> ", iOut );
  iOut1 = getShort();
  // wKE_R|[lggpt@CfBNg擾
  fprintf( stdout, "Input BetaRna-dir : " );
  getString( sBuf );

  // LŎ擾ݒl̊mFs
  fprintf( stdout, "\n" );
  fprintf( stdout, "Set to window func -> %d\n", iWF1 );
  fprintf( stdout, "Set to lower limit freq -> %lf\n", dLow1 );
  fprintf( stdout, "Set to upper limit freq -> %lf\n", dUp1 );
  fprintf( stdout, "Set to devide count -> %d\n", iOut1 );
  fprintf( stdout, "Set to BetaRna-dir -> <%s>\n", sBuf );
  fprintf( stdout, "Ready(y/n) : " );
  getString( sBuf1 );
  if( sBuf1[0] != 'y' )
    return;

  // fBNg̑݊mF
  {
    DIR* pDir = opendir( sBuf );
    if( pDir == NULL ) {
      fprintf( stdout, "ERROR : cannot find directory <%s>\n", sBuf );
      return;
    }
    closedir( pDir );
  }

  // o (Charact) R|[lg̐ݒlXV
  try {
    m_PowerInfo0->SetParam( iWF1, dLow1, dUp1, iOut1 );
  }
  catch(...) {
    fprintf( stdout, "Error in service port( PowerInfo ), check connection.\n" );
    return;
  }
  // ݒXVɔAwKE_R|[lg
  {
    // f[^fBNgpXAf[^o̓|[gɐݒ肷
    m_ParamDir.data = CORBA::string_dup( sBuf );
    // f[^fBNgpXAf[^o̓|[gAo͂
    m_ParamDirOut.write();
  }
}

/*
 @brief   ̃}CN͊Jn
 @return  bID (_sw肳ꂽA邢́AG[́A-1)
 @author  AdIn Research,Inc
 @date    2007/4/11
*/
int CUI::onStart( void )
{
  char  sTalker[64];
  char  sBuf[64];
  int   iRet;

RETRY_PROC:
  // bҖ̂̓͂𑣂
  fprintf( stdout, "Input Talker-Name, ? for detect : " );
  // ͂ꂽbҖ̂擾
  iRet = getString( sTalker );
  if( ! iRet )
    goto RETRY_PROC;

  // bҖ̂A'?'̏ꍇ
  if( sTalker[0] == '?' ) {
    // bID͖
    iRet = -1;
    // éAt@CۑȂ
    sBuf[0] = 0x00;
  }
  // bҖ̂A'?'ȊȌꍇ
  else {
    try {
      // bҊǗ (m_TalkerMng) R|[lgAbID擾
      iRet = m_TalkerMng0->GetID( sTalker );
    }
    catch(...) {
      fprintf( stdout, "Error in service port( TalkerMng ), check connection.\n" );
      iRet = -1;
    }
    // et@Cۑ邽߂̃fBNg쐬
    sprintf( sBuf, "voice/%s", sTalker );
  }

  // f[^o̓|[gAfBNgpXo͂
  m_Trigeger.data = CORBA::string_dup( sBuf );
  m_TrigegerOut.write();

  return iRet;
}

/*
 @brief   ̃t@C͊Jn
 @return  bID (_sw肳ꂽA邢́AG[́A-1)
 @author  AdIn Research,Inc
 @date    2007/4/11
*/
int CUI::onStartFile( void )
{
  char  sPath[256];
  char  sTalker[64];
  int   iRet;

RETRY_PROC:
  // bҖ̂̓͂𑣂
  fprintf( stdout, "Input Talker-Name, ? for detect : " );
  // ͂ꂽbҖ̂擾
  iRet = getString( sTalker );
  if( ! iRet )
    goto RETRY_PROC;

  // t@CpX̓͂𑣂
  fprintf( stdout, "Input Wave-file-path : " );
  // ͂ꂽt@CpX擾
  iRet = getString( sPath );
  if( ! iRet )
    goto RETRY_PROC;

  // bҖ̂A'?'̏ꍇ
  if( sTalker[0] == '?' ) {
    // bID́A
    iRet = -1;
  }
  // bҖ̂A'?'ȊȌꍇ
  else {
    try {
      // bҊǗ (m_TalkerMng) R|[lgAbID擾
      iRet = m_TalkerMng0->GetID( sTalker );
    }
    catch(...) {
      fprintf( stdout, "Error in service port( TalkerMng ), check connection.\n" );
      iRet = -1;
    }
  }

  // f[^o̓|[gAt@CpXo͂
  m_File.data = CORBA::string_dup( sPath );
  m_FileOut.write();

  return iRet;
}

/*
 @brief     f[^擾̏
 @param[in] iTalker bID
 @author    AdIn Research,Inc
 @date      2007/4/11
*/
bool CUI::onGetSound( int iTalker )
{
  int i;

  // TCYf[^̓|[g̃f[^ǂݍ݂s
  m_SizeIn.read();
  // f[^̓|[g̃f[^ǂݍ݂s
  m_WaveIn.read();

  // ǂݍ񂾉TCYf[^̏\
  fprintf( stdout, "\n\nReceived %d short.\n", m_Size.data );
  if( m_Size.data < 1024 ) {
    fprintf( stdout, "Too small voice data, egnored.\n" );
    return false;
  }

  // bIDAbIDf[^o̓|[gɁAݒ肷
  m_ID.data = iTalker;
  // ǂݍ񂾉TCYf[^ATCYf[^o̓|[gɐݒ肷
  m_SendSize.data = m_Size.data;
  // ǂݍ񂾉f[^Af[^o̓|[gɐݒ肷
  m_SendWave.data.length( m_SendSize.data );
  for( i = 0; i < m_SendSize.data; i++ )
    m_SendWave.data[i] = m_Wave.data[i];

  // bIDf[^o̓|[gAf[^o͂s
  m_IDOut.write();
  // TCYf[^o̓|[gAf[^o͂s
  m_SendSizeOut.write();
  // f[^o̓|[gAf[^o͂s
  m_SendWaveOut.write();

  return true;
}

/*
 @brief   ʃf[^擾̏
 @author  AdIn Research,Inc
 @date    2007/4/11
*/
void CUI::onGetResult()
{
  const char*   sName;

  // _ʃf[^̓|[g̓ǂݍ݂s
  m_ResultIDIn.read();
  // vxf[^̓|[g̓ǂݍ݂s
  m_TrustyIn.read();

  // T[rX|[goRŁAbҊǗ (TalkerMng) R|[lggAbIDAbҖ̂擾
  try {
    sName = m_TalkerMng0->GetName( (CORBA::Long)m_ResultID.data );
  }
  catch(...) {
    printf( "Error in service port, check connection.\n" );
    return;
  }

  // ʂɊւ\
  if( m_Trusty.data < 0.0 )
    // wK[h̏ꍇ
    fprintf( stdout, "BetaRna learned %s voice.\n", sName );
  else
    // _[h̏ꍇ
    fprintf( stdout, "BetaRna detected talker is %s, trust(%lf%%).\n", sName, m_Trusty.data );
}


extern "C"
{
 
  void CUIInit(RTC::Manager* manager)
  {
    RTC::Properties profile(cui_spec);
    manager->registerFactory(profile,
                             RTC::Create<CUI>,
                             RTC::Delete<CUI>);
  }
  
};



/*----------------- [eBeB֐ -----------------*/

/*
 @brief      [U̓ () 擾
 @param[out] sBuf ͂ꂽ
 @return     int ͂ꂽ̒
 @author     AdIn Research,Inc
 @date       2007/4/11
*/
int getString( char* sBuf )
{
  char*	ptr;

  fgets( sBuf, 255, stdin );

  ptr = strchr( sBuf, '\n' );
  if( ptr ) *ptr = 0x00;

  return strlen( sBuf );
}

/*
 @brief   [U̓ () 擾
 @return  short [U̓
 @author  AdIn Research,Inc
 @date    2007/4/11
*/
short getShort( void )
{
  char sBuf[64];

  fgets( sBuf, 63, stdin );
  return atoi( sBuf );
}

/*
 @brief   [U̓ (l) 擾
 @return  double [U̓
 @author  AdIn Research,Inc
 @date    2007/4/11
*/

double getDouble( void )
{
  char sBuf[64];

  fgets( sBuf, 63, stdin );
  return atof( sBuf );
}
