/*!
 @file   CPowerSpSp.cpp
 @brief  p[XyNgZo
 @author AdIn Research,Inc
 @date   2007/02/15
*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "CPowerSp.h"
#include "CommonFFT.h"

/*-------------- [eBeB֐` --------------*/
static int getCount( int iOrg );



/*
 @brief  RXgN^
 @author AdIn Research,Inc
 @date   2007/02/15
*/
CPowerSp::CPowerSp( void )
{
  m_dSamplingFreq = 20000.0;
  m_iWF = WLWF_HANNING;
  m_dLowerLim = 1500.0;
  m_dUpperLim = 8000.0;
  m_iDivCnt = 10;

  m_nPnt = 0;
  m_pdIn = m_pdLPF = m_pdWF = m_pdPower = m_pdSmPower = m_pdNormPower = NULL;
  m_pdReal = m_pdImage = NULL;

  if( ! loadParam() )
    saveParam();
}

/*
 @brief  fXgN^
 @author AdIn Research,Inc
 @date   2007/02/15
*/
CPowerSp::~CPowerSp(	void )
{
  clear();
}


/*
 @brief       ʂZo  
 @param[in]   nPnt     g`f[^TCY  (f[^z̔z)
 @param[in]   pPnt     g`f[^ (f[^z)
 @param[out]  piPCount ʂ̍ڐ (p[XyNg)
 @param[out]  ppdPower  (ʃf[^z)
 @return      int
 @note        ʂ̃f[^̈ (*ppdPower) ́AĂяoŊJ邱
 @author      AdIn Research,Inc
 @date        2007/02/15
*/
int CPowerSp::CalcPower( int nPnt, const short* pPnt, int* piPCount, double** ppdPower )
{
  int      i, j, iLowInd, iHighInd, iArea;
  double   dLPFFreq, dStep, dMin, dMax;
  double*  pdRet;

  if( ! nPnt || ! pPnt || ! piPCount || ! ppdPower )
    return 1;

  //----- t[Gϊ̎s ------------------------------------

  // Ɨ̈m
  if( ! allocAll( nPnt ) )
    return 2;

  // Ɨ̈ɁAl𕡐
  for( i = 0; i < nPnt; i++ )
    m_pdIn[i] = (double)pPnt[i];

  // [pXtB^[
  dLPFFreq = m_dSamplingFreq * WL_WDS_LPFRATE_DFL; // JbgItgZo
  if( ! LPF_Butterworth2( m_nPnt, m_pdIn, m_pdLPF, dLPFFreq, m_dSamplingFreq ) )
    return 3;

  // ֐Kp
  if( ! Windowing( m_iWF, m_nPnt, m_pdLPF, m_pdWF ) )
    return 4;

  // t[Gϊ (FFT) s
  if( ! FFT( m_nPnt, m_pdWF, m_pdReal, m_pdImage, m_pdPower ) )
    return 5;

  //----- ʎZo --------------------------------------------

  // ʑΏۋԂ̎Zo
  dStep = m_dSamplingFreq / m_nPnt;
  iLowInd = (int)( m_dLowerLim / dStep );
  iHighInd = (int)( m_dUpperLim / dStep );

  // ʃf[^̈̊m
  pdRet = (double*)malloc( sizeof( double ) * m_iDivCnt );
  if( ! pdRet )
    return 6;

  // p[XyNg̋ԕςZo
  {
    // ԒvZ
    iArea = ( iHighInd - iLowInd ) / m_iDivCnt;
    // eԂƂɁAԕςZo
    for( i = 0; i < m_iDivCnt; i++ ) {
      pdRet[i] = 0.0;
      // ԘaZo
      for( j = 0; j < iArea; j++ )
        pdRet[i] += m_pdPower[ iLowInd + i * iArea + j ];
      // ԕςZo
      pdRet[i] /= (double)iArea;
    }
  }

  // p[XyNg̋ԕς 0 ` 1 ̒lɐK
  {
    // ԕς́AőAŏl擾
    dMin = dMax = pdRet[0];
    for( i = 1; i < m_iDivCnt; i++ ) {
      if( dMin > pdRet[i] ) dMin = pdRet[i];
      if( dMax < pdRet[i] ) dMax = pdRet[i];
    }
    // K
    for( i = 0; i < m_iDivCnt; i++ )
      pdRet[i] = ( pdRet[i] - dMin ) / ( dMax - dMin );
  }

  // ߂lɑ
  *piPCount = m_iDivCnt; // ʂ̍ڐ
  *ppdPower = pdRet; // ʃf[^

  return 0;
}


/*
 @brief      ʌvZɎgpp^l擾
 @param[out] piWF ֐
 @param[out] pdLower g
 @param[out] pdUpper g
 @param[out] piDev p[XyNg
 @author     AdIn Research,Inc
 @date       2007/02/15
*/
void CPowerSp::GetParam( int* piWF, double* pdLower, double* pdUpper, int* piDev )
{
  *piWF = m_iWF;
  *pdLower = m_dLowerLim;
  *pdUpper = m_dUpperLim;
  *piDev = m_iDivCnt;
}

/*
 @brief      ʌvZɎgpp^lݒ
 @param[in]  iWF ֐
 @param[in]  dLower g
 @param[in]  dUpper g
 @param[in]  iDev p[XyNg
 @author     AdIn Research,Inc
 @date       2007/02/15
*/
void CPowerSp::SetParam( int iWF, double dLower, double dUpper, int iDev )
{
  m_iWF = iWF;
  m_dLowerLim = dLower;
  m_dUpperLim = dUpper;
  m_iDivCnt = iDev;

  saveParam();
}




#define  PS_PARAM_PATH    "ps.ini"
#define  PS_SAMPLE_FREQ   "SamplingFreq"
#define  PS_WIN_FUNC      "WindowFunction"
#define  PS_LOWER_FREQ    "LowerFreq"
#define  PS_UPPER_FREQ    "UpperFreq"
#define  PS_DIVIDE_NUM    "DivideNum"

/*
 @brief   ݒt@CAݒlǍ
 @return  bool
 @author  AdIn Research,Inc
 @date    2007/02/15
*/
bool CPowerSp::loadParam( void )
{
  FILE* fp;
  char sBuf[256];
  char* ptr;

  // ݒt@CJ
  fp = fopen( PS_PARAM_PATH, "rt" );
  if( ! fp )
    return false;

  // ݒt@CAsǂݍ
  while( fgets( sBuf, 255, fp ) ) {
    // '='T
    ptr = strchr( sBuf, '=' );
    if( ptr ) {
      // '='̎̕ (ݒ荀ڒlJnʒu) Ɉړ
      ptr++;
      // ݒ荀ڂATvOg̏ꍇ
      if( strstr( sBuf, PS_SAMPLE_FREQ ) )
        m_dSamplingFreq = atof( ptr );
      // ݒ荀ڂA֐ʂ̏ꍇ
      else if( strstr( sBuf, PS_WIN_FUNC ) )
        m_iWF = atoi( ptr );
      // ݒ荀ڂAg̏ꍇ
      else if( strstr( sBuf, PS_LOWER_FREQ ) )
        m_dLowerLim = atof( ptr );
      // ݒ荀ڂAg̏ꍇ
      else if( strstr( sBuf, PS_UPPER_FREQ ) )
        m_dUpperLim = atof( ptr );
      // ݒ荀ڂAp[XyNg̏ꍇ
      else if( strstr( sBuf, PS_DIVIDE_NUM ) )
        m_iDivCnt = atoi( ptr );
    }
  }

  // ݒt@C
  fclose( fp );
  return true;
}

/*
 @brief   ݒt@CɁAݒlۑ
 @return  bool
 @author  AdIn Research,Inc
 @date    2007/02/15
*/
bool CPowerSp::saveParam( void )
{
  FILE* fp;

  // ݒt@CJ
  fp = fopen( PS_PARAM_PATH, "wt" );
  if( ! fp ) {
    fprintf( stderr, "Write_file(%s) open error?\n", PS_PARAM_PATH );
    return false;
  }

  // TvOg̐ݒlo
  fprintf( fp, "%s=%lf\n", PS_SAMPLE_FREQ, m_dSamplingFreq );
  // ֐ʂ̐ݒlo
  fprintf( fp, "%s=%d\n", PS_WIN_FUNC, m_iWF );
  // g̐ݒlo
  fprintf( fp, "%s=%lf\n", PS_LOWER_FREQ, m_dLowerLim );
  // g̐ݒlo
  fprintf( fp, "%s=%lf\n", PS_UPPER_FREQ, m_dUpperLim );
  // p[XyNg̐ݒlo
  fprintf( fp, "%s=%d\n", PS_DIVIDE_NUM, m_iDivCnt );

  // ݒt@C
  fclose( fp );

  return true;
}


/*
 @brief     Ɨ̈mۂ
 @param[in] nPnt Ώۂ̃f[^
 @return    bool
 @author    AdIn Research,Inc
 @date      2007/02/15
*/
bool CPowerSp::allocAll( int nPnt )
{
  size_t tSize;

  clear();

  m_nPnt = getCount( nPnt );
  tSize = sizeof( double ) * m_nPnt;
  m_pdIn = (double*)malloc( tSize );
  m_pdLPF = (double*)malloc( tSize );
  m_pdWF = (double*)malloc( tSize );
  m_pdPower = (double*)malloc( tSize );
  m_pdSmPower = (double*)malloc( tSize );
  m_pdNormPower = (double*)malloc( tSize );
  m_pdReal = (double*)malloc( tSize );
  m_pdImage = (double*)malloc( tSize );
  if( ! m_pdIn || ! m_pdLPF || ! m_pdWF || ! m_pdPower || ! m_pdSmPower || ! m_pdNormPower ||
    ! m_pdReal || ! m_pdImage )
    return false;

  memset( (void*)m_pdIn, 0x00, tSize );
  memset( (void*)m_pdLPF, 0x00, tSize );
  memset( (void*)m_pdWF, 0x00, tSize );
  memset( (void*)m_pdPower, 0x00, tSize );
  memset( (void*)m_pdSmPower, 0x00, tSize );
  memset( (void*)m_pdNormPower, 0x00, tSize );
  memset( (void*)m_pdReal, 0x00, tSize );
  memset( (void*)m_pdImage, 0x00, tSize );

  return true;
}

/*
 @brief   ݒlNA
 @author  AdIn Research,Inc
 @date    2007/02/15
*/
void CPowerSp::clear( void )
{
  if( m_pdIn ) free( (void*)m_pdIn );
  if( m_pdLPF ) free( (void*)m_pdLPF );
  if( m_pdWF ) free( (void*)m_pdWF );
  if( m_pdPower ) free( (void*)m_pdPower );
  if( m_pdSmPower ) free( (void*)m_pdSmPower );
  if( m_pdNormPower ) free( (void*)m_pdNormPower );

  m_pdIn = m_pdLPF = m_pdWF = m_pdPower = m_pdSmPower = m_pdNormPower = NULL;

  if( m_pdReal ) free( (void*)m_pdReal );
  if( m_pdImage ) free( (void*)m_pdImage );

  m_pdReal = m_pdImage = NULL;
}


/*-------------- [eBeB֐ --------------*/

/*
 @brief     w肳ꂽ̃f[^̂ɁAKvƂȂƗ̈z̔z񐔂vZ
 @param[in] iOrg Ώۂ̃f[^
 @return    int Ɨ̈z̔z
 @author    AdIn Research,Inc
 @date      2007/02/15
*/
int getCount( int iOrg )
{
  int iRet = 1024;

  while( iRet < iOrg )
    iRet *= 2;

  return iRet;
}
