// -*- C++ -*-
/*!
 * @file  Charact.cpp
 * @brief Be charactaric component
 * $Date$
 *
 * $Id$
 */

#include "Charact.h"
#include "CPowerSp.h"

CPowerSp  g_cPS;


// Module specification
// <rtc-template block="module_spec">
static const char* charact_spec[] =
  {
    "implementation_id", "Charact",
    "type_name",         "Charact",
    "description",       "Be charactaric component",
    "version",           "0.1",
    "vendor",            "",
    "category",          "Generic",
    "activity_type",     "COMMUTATIVE",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

Charact::Charact(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_SizeIn("Size", m_Size),
    m_WaveIn("Wave", m_Wave),
    m_DataCountOut("DataCount", m_DataCount),
    m_DataAryOut("DataAry", m_DataAry),
    m_PowerInfoPort("PowerInfo"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Size", m_SizeIn);
  registerInPort("Wave", m_WaveIn);
  
  // Set OutPort buffer
  registerOutPort("DataCount", m_DataCountOut);
  registerOutPort("DataAry", m_DataAryOut);
  
  // Set service provider to Ports
  m_PowerInfoPort.registerProvider("PowerInfo0", "PowerInfo", m_PowerInfo0);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_PowerInfoPort);
  
  // </rtc-template>

}

Charact::~Charact()
{
}


/*
RTC::ReturnCode_t Charact::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Charact::onActivated(RTC::UniqueId ec_id)
{
  // f[^̓|[gɁA݂܂ĂꍇlāA
  // f[^̓|[gɑ΂āAxAǍݏs
  m_SizeIn.read();
  m_WaveIn.read();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Charact::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Charact::onExecute(RTC::UniqueId ec_id)
{
  int     i, j, iTmp, iRet, iPower;
  double* pdPower;

  // g`f[^TCY̓f[^|[gɁAVȔg`f[^͂ꂽꍇ
  if( m_SizeIn.isNew() ) {
    // g`f[^TCY̓f[^|[g̓ǂݍݏs
    m_SizeIn.read();
    // g`f[^̓f[^|[g̓ǂݍݏs
    m_WaveIn.read();

    // ͂ꂽg`f[^̃TCY`FbN
    fprintf( stdout, "Inport data size = %ld\n\n", m_Size.data );
    if( m_Size.data < 1024 ) {
      fprintf( stdout, "Too small inport data, egnored.\n" );
      return RTC::RTC_OK;
    }

    // g`f[^ʉ
    iRet = g_cPS.CalcPower( (int)m_Size.data, &( m_Wave.data[0] ), &iPower, &pdPower );
    if( iRet ) {
      // ʉs
      fprintf( stdout, "Error in CalcPower(), err = %d.\n", iRet );
      fprintf( stdout, "I dont send to next comp.\n" );
    }
    else {
      // ʉ

      // ʂAȈՃOt\
      {
        fprintf( stdout, "0.0       0.5       1.0\n" );  // ȈՃOt̖ڐ\
        // ʂ̊eڂAɖ_Ot
        for( i = 0; i < iPower; i++ ) {
          // ʍڂ̃CfbNX\
          fprintf( stdout, "[%02d]", i );
          // ʍڂ̖_Ot̒vZ
          iTmp = (int)( pdPower[i] * 20.0 );
          // ʍڂ̖_Ot ("******"`) o
          for( j = 0; j < iTmp; j++ )
            fputc( '*', stdout );
          // ̓ʍڕ\̈Ɉړ
          fputc( '\n', stdout );
        }
      }

      // ʂ̍ڐAf[^o̓|[gɐݒ
      m_DataCount.data = iPower;
      // ʂAf[^o̓|[gɐݒ
      m_DataAry.data.length( iPower );
      for( i = 0; i < iPower; i++ )
        m_DataAry.data[i] = pdPower[i];

      // ʂAf[^o̓|[gAo
      m_DataAryOut.write();
      // ʂ̍ڐAf[^o̓|[gAo
      m_DataCountOut.write();

      // ㏈
      free( (void*)pdPower );
    }

    fprintf( stdout, "\n\n" );
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Charact::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Charact::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CharactInit(RTC::Manager* manager)
  {
    RTC::Properties profile(charact_spec);
    manager->registerFactory(profile,
                             RTC::Create<Charact>,
                             RTC::Delete<Charact>);
  }
  
};


