// -*- C++ -*-
/*!
 * @file  GUI.h
 * @brief Demo GUI component
 * @date  $Date$
 *
 * $Id$
 */

#ifndef GUI_H
#define GUI_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "TalkerMngStub.h"
#include "PowerInfoStub.h"
#include "BetaRnaStub.h"


// </rtc-template>

using namespace RTC;

class GUI
  : public RTC::DataFlowComponentBase
{
 public:
  GUI(RTC::Manager* manager);
  ~GUI();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
  // virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


// protected:
public:
  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedLong m_Size;
  InPort<TimedLong,RTC::RingBuffer> m_SizeIn;
  TimedShortSeq m_Wave;
  InPort<TimedShortSeq> m_WaveIn;
  TimedShort m_DataCount;
  InPort<TimedShort,RTC::RingBuffer> m_DataCountIn;
  TimedDoubleSeq m_DataAry;
  InPort<TimedDoubleSeq> m_DataAryIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedString m_Trigeger;
  OutPort<TimedString> m_TrigegerOut;
  TimedString m_File;
  OutPort<TimedString> m_FileOut;
  TimedLong m_SendSize;
  OutPort<TimedLong> m_SendSizeOut;
  TimedShortSeq m_SendWave;
  OutPort<TimedShortSeq> m_SendWaveOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_TalkerMngPort;
  RTC::CorbaPort m_PowerInfoPort;
  RTC::CorbaPort m_BetaRnaPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  RTC::CorbaConsumer<TalkerMng> m_TalkerMng0;
  RTC::CorbaConsumer<PowerInfo> m_PowerInfo0;
  RTC::CorbaConsumer<BetaRna> m_BetaRna0;
  
  // </rtc-template>

 private:
  int dummy;
};


extern "C"
{
  void GUIInit(RTC::Manager* manager);
};

#endif // GUI_H
