// -*- C++ -*-
/*!
 * @file  Result.cpp
 * @brief Result display component
 * $Date$
 *
 * $Id$
 */

#include "Result.h"

// Module specification
// <rtc-template block="module_spec">
static const char* result_spec[] =
  {
    "implementation_id", "Result",
    "type_name",         "Result",
    "description",       "Result display component",
    "version",           "0.1",
    "vendor",            "",
    "category",          "Generic",
    "activity_type",     "COMMUTATIVE",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

Result::Result(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ResultIDIn("ResultID", m_ResultID),
    m_TrustyIn("Trusty", m_Trusty),
    m_TalkerMngPort("TalkerMng"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ResultID", m_ResultIDIn);
  registerInPort("Trusty", m_TrustyIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_TalkerMngPort.registerConsumer("TalkerMng0", "TalkerMng", m_TalkerMng0);
  
  // Set CORBA Service Ports
  registerPort(m_TalkerMngPort);
  
  // </rtc-template>

}

Result::~Result()
{
}

/*
RTC::ReturnCode_t Result::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  
  // </rtc-template>
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Result::onActivated(RTC::UniqueId ec_id)
{
  // f[^̓|[gɁA݂Ăꍇ̂
  // f[^̓|[gɑ΂āAxAǂݍݏs
  m_ResultIDIn.read();
  m_TrustyIn.read();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Result::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Result::onExecute(RTC::UniqueId ec_id)
{
  const char* sName;

  // _ʃf[^ (=bID) ̓|[gɁAVf[^͂ꂽꍇ
  if( m_ResultIDIn.isNew() ) {
    // _ʃf[^̓|[g̓ǂݍ݂s
    m_ResultIDIn.read();
    // vxf[^̓|[g̓ǂݍ݂s
    m_TrustyIn.read();

    // T[rX|[goRŁAbҊǗ (TalkerMng) R|[lggAbIDAbҖ̂擾
    try {
      sName = m_TalkerMng0->GetName( (CORBA::Long)m_ResultID.data );
    }
    catch(...) {
      printf( "Error in service port, check connection.\n" );
      return RTC::RTC_OK;
    }

    // ʂɊւ\
    if( m_Trusty.data < 0.0 )
      // wK[h̏ꍇ
      fprintf( stdout, "RNA learned %s data\n\n", sName );
    else
      // _[h̏ꍇ
      fprintf( stdout, "RNA detected, voice is %s(%lf%%).\n\n", sName, m_Trusty.data );
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Result::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Result::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ResultInit(RTC::Manager* manager)
  {
    RTC::Properties profile(result_spec);
    manager->registerFactory(profile,
                             RTC::Create<Result>,
                             RTC::Delete<Result>);
  }
  
};


