// -*- C++ -*-
/*!
 * @file  VoiceInMic.cpp
 * @brief Voice input by mic component
 * $Date$
 *
 * $Id$
 */

#include "VoiceInMic.h"
#include "CJuliusRecv.h"

static CJuliusRecv  l_cJR;

// Module specification
// <rtc-template block="module_spec">
static const char* voiceinmic_spec[] =
  {
    "implementation_id", "VoiceInMic",
    "type_name",         "VoiceInMic",
    "description",       "Voice input by mic component",
    "version",           "0.1",
    "vendor",            "",
    "category",          "Generic",
    "activity_type",     "COMMUTATIVE",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

VoiceInMic::VoiceInMic(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_TriggerIn("Trigger", m_Trigger),
    m_SizeOut("Size", m_Size),
    m_WaveOut("Wave", m_Wave),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Trigger", m_TriggerIn);
  
  // Set OutPort buffer
  registerOutPort("Size", m_SizeOut);
  registerOutPort("Wave", m_WaveOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

VoiceInMic::~VoiceInMic()
{
}


/*
RTC::ReturnCode_t VoiceInMic::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t VoiceInMic::onActivated(RTC::UniqueId ec_id)
{
  // gK[f[^̓|[gɁA݂݂\邽߁A
  // gK[f[^̓|[g̃f[^ǂݍݏs
  m_TriggerIn.read();

  // }CN̓c[̏s
  if( ! l_cJR.Init() )
    fprintf( stdout, "Fatal error, failed to connect adintool.\n" );

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t VoiceInMic::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t VoiceInMic::onExecute(RTC::UniqueId ec_id)
{
  const short* pBuf;
  int          i, iSize;

  // gK[f[^̓|[gɁA̓f[^݂ꍇ
  if( m_TriggerIn.isNew() ) {

    // gK[f[^̓|[g̃f[^(f[^̕ۑfBNgpX)ǂݍ
    m_TriggerIn.read();

    // ̓c[ɁAf[^̕ۑfBNgpXn
    l_cJR.SetDir( (char*)m_Trigger.data );
    fprintf( stdout, "Now recording..." );
    fflush( stdout );

    // ̓c[ɁA͊Jn𑣂āA͂ꂽf[^擾
    pBuf = l_cJR.Read();

    // ͂ꂽf[^̃f[^TCY(16rbgf[^Zbg̐)擾
    iSize = l_cJR.GetSize();
    fprintf( stdout, "Received %d sets of 16bits data.\n", iSize );

    // ͂ꂽf[^TCYA[łꍇ
    if( iSize >= 1024 ) {
      // f[^ó[|[gɁAf[^TCY(16rbgf[^Zbg̐)ݒ肷
      m_Size.data = iSize;

      // f[^ó[|[gɁAf[^ݒ肷
      m_Wave.data.length( iSize );
      for( i = 0; i < iSize; i++ )
        m_Wave.data[i] = pBuf[i];

      // f[^TCY(16rbgf[^Zbg̐)o͂
      m_WaveOut.write();

      // f[^o͂
      m_SizeOut.write();
    }

    // ͂ꂽf[^TCYAꍇ
    else {
      fprintf( stdout, "Too small wave data, ignore.\n" );
    }

    fprintf( stdout, "\n" );
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t VoiceInMic::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t VoiceInMic::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void VoiceInMicInit(RTC::Manager* manager)
  {
    RTC::Properties profile(voiceinmic_spec);
    manager->registerFactory(profile,
                             RTC::Create<VoiceInMic>,
                             RTC::Delete<VoiceInMic>);
  }
  
};


