/******************************************************************************
 *      (C)Copyright        AdIn Research, Inc.         1991-1992.            *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rnaF0def.h
 *
 *  Note       : Beta-RNA Data File Loader Developer's File
 *
 * $Revision: 1.43 $
 * $Author: fujimaki $
 * $Date: 1999/11/16 06:36:46 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(_RNAF00DEF_H_)
#define _RNAF00DEF_H_

#include "rna00def.h"
#include "rnaF0usr.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/********** Symbols **********/
#define BLOCK_BEGIN                 '{'
#define BLOCK_END                   '}'
#define LEFT_PAREN                  '('
#define RIGHT_PAREN                 ')'
#define STRING_BEGIN                '\"'
#define STRING_END                  (STRING_BEGIN)
#define RID_BEGIN                   '#'
#define DIM_BEGIN                   '['
#define DIM_DELIM                   ','
#define DIM_END                     ']'
#define DIM_MAX                     (3)
#define ESCAPE_CHAR_PREFIX          '\\'
#define COMMENT_CHAR                ';'
#define ANARULE_PREFIX_STR          "A"
#define ANARULE_PREFIX_LEN          (1)
#define DIGRULE_PREFIX_STR          "D"
#define DIGRULE_PREFIX_LEN          (1)
#define RULE_WEIGHT_DELIM           ':'

#define DEFAULT_FIO_INPUT           (stdin)
#define DEFAULT_FIO_INPUT_NAME      "stdin"
#define DEFAULT_FIO_OUTPUT          (stdout)
#define DEFAULT_FIO_OUTPUT_NAME     "stdout"
#define NAME_MAXLEN                 (31)
#define STRING_MAXLEN               (127)
#define TOKEN_MAXLEN                (MAX(NAME_MAXLEN, STRING_MAXLEN))
#define TOKEN_STACK_SIZE            (5)
#define DEFAULT_TAB_WIDTH           (4)

/********** Miscellaneous **********/
#define newline(p)                  (putc('\n', (p)))

/********** Character Predicates **********/
#define nameFirstP(c) \
    ((isascii(c) && isalpha(c)) || (c) == '_')
#define nameRestP(c) \
    ((isascii(c) && isalpha(c)) || isdigit(c) || (c) == '_')
#define digitP(c) \
    (isascii(c) && isdigit(c))
#define digitBeginP(c) \
    ((c) == '+' || (c) == '-' || (digitP(c)))

/********** Token Type **********/
typedef enum {
    /***** reserved (system) *****/
    TK_NOTHING = 0,
    /***** I/O directives *****/
    TK_BLK_BEGIN,                       /* { */
    TK_BLK_END,                         /* } */
    TK_LPAREN,                          /* ( */
    TK_RPAREN,                          /* ) */
    /***** values *****/
    TK_REAL,                            /* ([0-9]*\.[0-9]+|[0-9]+\.[0-9]*) */
    TK_INT,                             /* [0-9]+ */
    TK_RID,                             /* #[0-9]+ */
    TK_STR,                             /* \".*\" */
    TK_SYM,                             /* [_A-Za-z]+[0-9_A-Za-z]* */
    TK_DIM,                             /* \[[1-9]+[0-9]*(,[1-9]+[0-9]*)\] */
    /***** normalization *****/
    TK_NTBL_BLK,                        /* Normalization */
    TK_IN_BLK,                          /* I */
    TK_OUT_BLK,                         /* O */
    /***** matching *****/
    TK_ANARULE_PREFIX,                  /* A */
    TK_DIGRULE_PREFIX,                  /* D */
    TK_RWEIGHT_DELIM,                   /* : */
    TK_TYPE_BLK,                        /* Type */
    TK_RULE_BLK,                        /* MatchRule */
    TK_MIN_NAM,                         /* Min */
    TK_ADDMEAN_NAM,                     /* Addmean */
    TK_MULMEAN_NAM,                     /* Mulmean */
    TK_RULE_NAM,                        /* Rule */
    TK_LINEAR_NAM,                      /* Linear */
    TK_RAW_NAM,                         /* Raw */
    TK_WEIGHT_NAM,                      /* Weight */
    TK_SRAW_NAM,                        /* SRaw */
    TK_SWEIGHT_NAM,                     /* SWeight */
    TK_SWINT_NAM,                       /* SWint */
    TK_SWEXT_NAM,                       /* SWext */
    TK_ADD_OPR,                         /* Add */
    TK_WADD_OPR,                        /* WAdd */
    TK_MULT_OPR,                        /* Mul */
    TK_WMULT_OPR,                       /* WMul */
    TK_NOT_OPR,                         /* Not */
    /***** sensor set *****/
    TK_SSET_BLK,                        /* SensorSet */
    TK_ANA_BLK,                         /* Analogue */
    TK_DIG_BLK,                         /* Digital */
    /***** pattern table *****/
    TK_PTBL_BLK,                        /* PatternTable */
    TK_PSET_BLK,                        /* PatternSet */
    TK_MF_BLK,                          /* F */
    TK_DVAL_BLK,                        /* D */
    TK_OUTSSET_BLK,                     /* OutputSensor */
    TK_OUTCOND_BLK,                     /* OutputCond */
    /***** pattern group *****/
    TK_PGRP_BLK,                        /* PatternGroup */
    TK_FUSION_BLK,                      /* Fusion */
    TK_THRESHOLD_BLK,                   /* Threshold */
    TK_DIGON_NAM,                       /* ON */
    TK_DIGOFF_NAM,                      /* OFF */
    /***** learning *****/
    TK_LBASIC_BLK,
    TK_LEXTEND_BLK,
    TK_LWEIGHT_BLK,
    TK_LSTATUS_BLK,
    TK_LHIST_BLK,
    TK_ACTIVE_NAM,
    TK_HISTORY_NAM,
    TK_LEARN_NAM,
    TK_NOCLRHST_NAM,
    TK_VAREXT_NAM,
    TK_VARFIX_NAM,
    TK_EXTEND_NAM,
    TK_SMALL_NAM,
    TK_COMPENS_NAM,
    TK_NOINHERIT_NAM,
    TK_ADDVAR_NAM,
    /***** data table *****/
    TK_DTBL_BLK,
    TK_DSET_BLK,
    /***** condition table *****/
    TK_CTBL_BLK,
    TK_CSET_BLK,
    /***** condition table *****/
    TK_XXXXXXXXXX               /* dummy */
} TOKENTYPE;

/********** Token Attribute **********/
typedef unsigned short  TOKENATTR;
#define TA_BLK          (0x0001)        /* block symbol */
#define TA_SYM          (0x0002)        /* conventional symbol */
#define TA_REAL         (0x0004)        /* real number */
#define TA_INT          (0x0008)        /* integer number */
#define TA_ID           (0x0010)        /* ID */
#define TA_STR          (0x0020)        /* quoted string */
#define TA_DIM          (0x0040)        /* dimension */
#define TA_CTL          (0x0080)        /* control flow, etc. */
#define TA_ROP          (0x0100)        /* rule operator */

/********** Number Type **********/
typedef enum {
    NUM_GUESS,                          /* guess number type */
    NUM_REAL,                           /* real */
    NUM_INT,                            /* integer */
    NUM_RID                             /* ID */
} NUMTYPE;

/********** Token **********/
typedef struct {
    TOKENTYPE type;                     /* token type */
    TOKENATTR attr;                     /* token attribute */
    char      str[TOKEN_MAXLEN + 1];    /* token string */
    union {
        double real;                    /* real number */
        long   integer;                 /* integer, analogue data */
        RID    rid;                     /* ID */
        RDIG   rdig;                    /* digital data */
        RNUM   dim[DIM_MAX];            /* dimension */
    } value;
} TOKEN;

/********** for rcGetLine() **********/
typedef struct {
    int   len;                          /* buffer length */
    char *sBuf;                         /* buffer */
} LINE;

/********** I/O Configuration **********/
#define CF_ENTRY_MAXLEN         (16)

typedef struct {
    char sReal[CF_ENTRY_MAXLEN];
    char sNorm[CF_ENTRY_MAXLEN];
} CONFIG;

/***** format of real number *****/
#define CF_REAL             "FioFormatReal"
#define CF_FORMAT_REAL      "%.8e"
/***** format of normalization number format (secret function ^_^;) *****/
#define CF_NORM             "FioFormatNorm"
#define CF_FORMAT_NORM      "%lu"

extern CONFIG config_f0_;

/********** Reader Information **********/
typedef struct {
    char            *sName;                             /* section name */
    Ferr PROTOTYPE((PUBLIC *pfnReader), (STREAM *));    /* reader */
} READTABLE;

/********** Writer Information **********/
typedef struct {
    RNUM                    rnData;                         /* number of data */
    Ferr PROTOTYPE((PUBLIC *pfnWriter), (STREAM *, RNUM));  /* writer */
} WRITETABLE;

/********** Label Functions **********/
extern Ferr PROTOTYPE((PUBLIC *pfnSetLabel_f0_), (RCAT, RID *, RNUM, char *));
extern Ferr PROTOTYPE((PUBLIC *pfnGetLabel_f0_), (RCAT, RID *, RNUM, char *));

/********** Learning Functions **********/
extern Ferr PROTOTYPE((*pfnReadLearnBasic_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnReadLearnExtend_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnReadLearnWeight_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnReadLearnStatus_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnReadLearnHist_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnWriteLearnBasic_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnWriteLearnExtend_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnWriteLearnWeight_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnWriteLearnStatus_f0_),
                      (STREAM *, RCAT, RID, RID, RID));
extern Ferr PROTOTYPE((*pfnWriteLearnHist_f0_),
                      (STREAM *, RCAT, RID, RID, RID));

/********** I/O Function Prototype **********/
extern Ferr  PROTOTYPE(rcReadConfigFile, (void));
extern char *PROTOTYPE(streamInfo, (STREAM *));
extern char *PROTOTYPE(streamInfo2, (STREAM *)); /* ask `selfish Ken' why */
extern void  PROTOTYPE(resetIndent, (void));
extern void  PROTOTYPE(indent, (int));
extern void  PROTOTYPE(tabulation, (STREAM *));
extern Ferr  PROTOTYPE(rcOpenStream, (const char *, FOPEN_MODE, STREAM *));
extern Ferr  PROTOTYPE(rcCloseStream, (STREAM *));
extern Ferr  PROTOTYPE(rcMatchName, (char *, char *));
extern Ferr  PROTOTYPE(rcMatchNameN, (char *, char *, int));
extern Ferr  PROTOTYPE(rcGetToken, (STREAM *, TOKEN *));
extern Ferr  PROTOTYPE(rcUngetToken, (STREAM *, TOKEN *));
extern Ferr  PROTOTYPE(rcExpectToken, (STREAM *, TOKENTYPE, TOKEN *));
extern Ferr  PROTOTYPE(rcGetLine, (STREAM *));
extern Ferr  PROTOTYPE(rcGetStreamPosition, (STREAM *, long *));
extern Ferr  PROTOTYPE(rcSetStreamPosition, (STREAM *, long));
extern Ferr  PROTOTYPE(rcSkipBlock, (STREAM *));
extern Ferr  PROTOTYPE(rcGetLineNumber, (int *));
extern Ferr  PROTOTYPE(rcReadMembership, (STREAM *, RMF *, RANA *));
extern Ferr  PROTOTYPE(rcReadWeight, (STREAM *, RANA *));
extern Ferr  PROTOTYPE(rcReadOutData, (STREAM *, RID *));
extern Ferr  PROTOTYPE(rcReadWeight, (STREAM *, RANA *));
extern Ferr  PROTOTYPE(rcReadMatchType, (STREAM *, RMTYPE *));
extern Ferr  PROTOTYPE(rcReadDigForm, (STREAM *, RDIG *, RANA *));
extern Ferr  PROTOTYPE(rcReader, (STREAM *, READTABLE *, int));

extern Ferr  PROTOTYPE(rcWriteLabel, (STREAM *, char *));
extern Ferr  PROTOTYPE(rcWriteMembership, (STREAM *, RMF *, RANA));
extern Ferr  PROTOTYPE(rcWriteMatchType, (STREAM *, RMTYPE));
extern Ferr  PROTOTYPE(rcWriter, (STREAM *, WRITETABLE *, int));

/***** Model01 *****/
extern Ferr  PROTOTYPE(rcReadLearnParam, (STREAM *, RCAT, RID, RID, RID));
extern Ferr  PROTOTYPE(rcWriteLearnParam, (STREAM *, RCAT, RID, RID, RID));

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* not _RNAF00DEF_H_ */
