/******************************************************************************
 *      Copyright      Advanced Intelligence Research, Inc.   1991-1992       *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rnaS2usr.h
 *
 *  Note       : Beta-RNA Learnig Routine (Type 2)
 *
 * $Revision: 2.24 $
 * $Author: fujimaki $
 * $Date: 1999/11/05 03:03:48 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(RNAS2USR_H)
#define RNAS2USR_H

#include "rnaS0usr.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/**STDEXT_BEGIN**/
/*------------------------ Struct for Output System Parameter ---------------*/
typedef struct {
    unsigned long  ulMode;
    unsigned long  ulStat;
    RLMFP         *prlm;
    RLEXT         *prle;
    RLWEIGHT      *prlw;
} RS2OUT;

/*------------------------ Struct for Beta-RNA Simurator Type2 --------------*/
typedef struct {
    unsigned long  ulMode;
    unsigned long  ulStat;
    RLMFP*         prlm;
    RLEXT*         prle;
    RLWEIGHT*      prlw;
    int            iOutMode;
    RNUM           rnActRes;
    RS2OUT*        prs2out;
    RNUM           rnIn;
    double*        pdMaxIn;
    double*        pdMinIn;
    RID            riPgrp;
    RID            riSset;
    RID*           priPtbl;
    RNUM           rnOut;
    double*        pdMaxOut;
    double*        pdMinOut;
    RID            riPgrpOut;
    RID            riSsetOut;
    RID            riPtblOut;
    RANA           raSysMax;
    unsigned long  ulMagic;
    RMD            rgM2Thr;             /* 96.02.06 */
    double*        pdM2Param;           /* 96.02.06 */
} RS2DATA;

/*-------------------- Prototype (Prop. EXPORTal Functions)------------------*/
EXPORT Ferr PUBLIC PROTOTYPE(RS2_Init, (RS2DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_Set,  (RS2DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_Free, (RS2DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_FreeKernelData, (RS2DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_Exec, (RS2DATA *, double *, TSIG));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_Prod, (RS2DATA *, double *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_GetLearnInfo, (int, LTYPE *, RNUM *, RID *, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_GetMatchResult, (RS2DATA *, RNUM, RMD *, double *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_SetInput, (RS2DATA *, double *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_SetDataSelection, (RS2DATA *, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_GetDataSelection, (RS2DATA *, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_GetMin2Param, (RS2DATA*)); /* 96.02.06 */
EXPORT Ferr PUBLIC PROTOTYPE(RS2_Min2Prod, (RS2DATA*, double*)); /* 96.02.06 */
EXPORT Ferr PUBLIC PROTOTYPE(rs2SetLearnParam, (RS2DATA *));
/**STDEXT_END**/

EXPORT Ferr PUBLIC PROTOTYPE(RS2_PrintLearnResult, (unsigned long, RS2DATA *,
                                             double *, FILE *, int, Ferr));
EXPORT Ferr PUBLIC PROTOTYPE(RS2_PrintInferResult, (unsigned long, RS2DATA *,
                                             double *, RBOOL, FILE *, int, Ferr));

/*-------------------- Prototype (EXPORTal Functions)------------------------*/
EXPORT Ferr PUBLIC PROTOTYPE(RS2LibInit, (char *, unsigned long, unsigned long,
                                   RLMFP *, RLEXT *, RLWEIGHT *,
                                   int, RNUM, RS2OUT *, RANA, RMD,
                                   RS2DATA **, STREAM *)); /* 96.02.06 */
EXPORT Ferr PUBLIC PROTOTYPE(RS2LibSet,  (char *, RID, RID,
                                   unsigned long, unsigned long,
                                   RLMFP *, RLEXT *, RLWEIGHT *,
                                   int, RNUM, RMD,
                                   RS2OUT *, RS2DATA **, STREAM *)); /* 96.02.06 */
EXPORT Ferr PUBLIC PROTOTYPE(RS2LibFree, (RS2DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2LibExec, (RS2DATA *, double *, STREAM *));
EXPORT Ferr PUBLIC PROTOTYPE(RS2LibProd, (RS2DATA *, double *, STREAM *));

/*-------------------- Macro (EXPORTal Functions) ---------------------------*/
#define RS2LibSetInfer(sFile, iPg, oPg, ulM, iOMode, rnAct, pprs2, ppFile) \
       (RS2LibSet((sFile), (iPg), (oPg), (ulM), 0L, \
                  (RLMFP *)NULL, (RLEXT *)NULL, (RLWEIGHT *)NULL, \
                  (iOMode), (rnAct), (RS2OUT *)NULL, (pprs2), (ppFile)))

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* RNAS2USR_H */
